#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_InterlaceIntesnsity; // 0.01
uniform float u_DropoutIntensity; // 0.01
uniform float u_DisplaceIntesnsity; // 1.0
uniform float u_RGBIntesnsity; // 1.0
uniform float u_Coeff1; // 0.33

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

float rand(float n){return fract(sin(n) * 43758.5453123);}

float noise(float p){
    float fl = floor(p);
    float fc = fract(p);
    return mix(rand(fl), rand(fl + 1.0), fc);
}

float blockyNoise(vec2 uv, float threshold, float scale, float seed)
{
    float scroll = floor(u_Time + sin(11.0 *  u_Time) + sin(u_Time) ) * 0.77;
    vec2 noiseUV = uv.yy / scale + scroll;
    float noise2 = texture(u_NoiseTexture, noiseUV).r;

    float id = floor( noise2 * 20.0);
    id = noise(id + seed) - 0.5;


    if ( abs(id) > threshold ) {
        id = 0.0;
    }

    return id;
}

void main() {
    float rgbIntesnsity = (0.1 + 0.1 * sin(u_Time* 3.7)) * map(u_RGBIntesnsity, 0.0, 1.0, 0.1, 4.0);
    float displaceIntesnsity = (0.2 +  0.3 * pow( sin(u_Time * 1.2), 5.0)) * map(u_DisplaceIntesnsity, 0.0, 1.0, 0.1, 4.0);
    float interlaceIntesnsity = map(u_InterlaceIntesnsity, 0.0, 1.0, 0.001, 1.0);
    float dropoutIntensity = map(u_DropoutIntensity, 0.0, 1.0, 0.001, 1.0);


    vec2 uv = gl_FragCoord.xy/u_Resolution.xy;

    float displace = blockyNoise(uv + vec2(uv.y, 0.0), displaceIntesnsity, 25.0, 66.6);
    displace *= blockyNoise(uv.yx + vec2(0.0, uv.x), displaceIntesnsity, 111.0, 13.7);

    uv.x += displace ;

    vec2 offs = 0.1 * vec2(blockyNoise(uv.xy + vec2(uv.y, 0.0), rgbIntesnsity, 65.0, 341.0), 0.0);

    vec2 colr = texture(u_Texture, uv-offs).ra;
    vec2 colg = texture(u_Texture, uv).ga;
    vec2 colb = texture(u_Texture, uv +offs).ba;


    float line = fract(gl_FragCoord.y / 3.0);
    vec3 mask = vec3(3.0, 0.0, 0.0);

    if (line > u_Coeff1) {
        mask = vec3(0.0, 3.0, 0.0);
    }

    if (line > 1.0 - u_Coeff1) {
        mask = vec3(0.0, 0.0, 3.0);
    }


    float maskNoise = blockyNoise(uv, interlaceIntesnsity, 90.0, u_Time) * max(displace, offs.x);

    maskNoise = 1.0 - maskNoise;

    if ( maskNoise == 1.0) {
        mask = vec3(1.0);
    }

    float dropout = blockyNoise(uv, dropoutIntensity, 11.0, u_Time) * blockyNoise(uv.yx, dropoutIntensity, 90.0, u_Time);
    mask *= (1.0 - 5.0 * dropout);


    o_FragColor = vec4(mask * vec3(colr.x, colg.x, colb.x), (colr.y + colg.y + colb.y) / 3.0);
}
