#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_GlitchAmplitude; // 0.2
uniform float u_GlitchNarrowness; // 4.0
uniform float u_GlitchBlockiness; // 2.0
uniform float u_GlitchMinimizer; // 5.0

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

float rand(vec2 p)
{
    float t = floor(u_Time * 20.) / 10.;
    return fract(sin(dot(p, vec2(t * 12.9898, t * 78.233))) * 43758.5453);
}

float noise(vec2 uv, float blockiness)
{
    vec2 lv = fract(uv);
    vec2 id = floor(uv);

    float n1 = rand(id);
    float n2 = rand(id+vec2(1,0));
    float n3 = rand(id+vec2(0,1));
    float n4 = rand(id+vec2(1,1));

    vec2 u = smoothstep(0.0, 1.0 + blockiness, lv);

    return mix(mix(n1, n2, u.x), mix(n3, n4, u.x), u.y);
}

float fbm(vec2 uv, int count, float blockiness, float complexity)
{
    float val = 0.0;
    float amp = 0.5;

    while(count != 0)
    {
        val += amp * noise(uv, blockiness);
        amp *= 0.5;
        uv *= complexity;
        count--;
    }

    return val;
}

float glitchAmplitude = 0.2;
float glitchNarrowness = 4.0;
float glitchBlockiness = 2.0;
float glitchMinimizer = 5.0; // decrease this

void main() {
    glitchAmplitude = map(u_GlitchAmplitude, 0.0, 1.0, 0.1, 2.0);
    glitchNarrowness = map(u_GlitchNarrowness, 0.0, 1.0, 0.1, 10.0);
    glitchBlockiness = map(u_GlitchBlockiness, 0.0, 1.0, 0.1, 10.0);
    glitchMinimizer = map(u_GlitchMinimizer, 0.0, 1.0, 0.1, 7.0);

    // Normalized pixel coordinates (from 0 to 1)
    vec2 uv = gl_FragCoord.xy/u_Resolution.xy;
    vec2 a = vec2(uv.x * (u_Resolution.x / u_Resolution.y), uv.y);
    vec2 uv2 = vec2(a.x / u_Resolution.x, exp(a.y));
    vec2 id = floor(uv * 8.0);
    //id.x /= floor(texture(u_Texture, vec2(id / 8.0)).r * 8.0);

    // Generate shift amplitude
    float shift = glitchAmplitude * pow(fbm(uv2, int(rand(id) * 6.), glitchBlockiness, glitchNarrowness), glitchMinimizer);

    // Create a scanline effect
    float scanline = abs(cos(uv.y * 400.));
    scanline = smoothstep(0.0, 2.0, scanline);
    shift = smoothstep(0.00001, 0.2, shift);

    // Apply glitch and RGB shift
    vec2 colR = texture(u_Texture, vec2(uv.x + shift, uv.y)).ra;
    vec2 colG = texture(u_Texture, vec2(uv.x - shift, uv.y)).ga;
    vec2 colB = texture(u_Texture, vec2(uv.x - shift, uv.y)).ba;

    // Mix with the scanline effect
    vec3 f = vec3(colR.x * (1. - shift), colG.x * (1. - shift) + rand(id) * shift, colB.x  * (1. - shift)) - (0.1 * scanline);

    // Output to screen
    o_FragColor = vec4(f, (colR.y + colG.y + colB.y) / 3.0);
}
