#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // repeat
uniform sampler2D u_NoiseTexture0;
uniform sampler2D u_NoiseTexture1;

uniform float u_Coeff1; // 0.005
uniform float u_Coeff2; // 0.02
uniform float u_Coeff3; // 0.25

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

#define pi 3.1415926


vec4 colorSplit(vec2 uv, vec2 s)
{
    vec4 color;
    vec2 r = texture(u_Texture, uv - s).ra;
    vec2 g = texture(u_Texture, uv    ).ga;
    vec2 b = texture(u_Texture, uv + s).ba;

    color.r = r.x;
    color.g = g.x;
    color.b = b.x;
    color.a = (r.y + b.y + g.y) / 3.0;

    return color;
}

vec2 interlace(vec2 uv, float s)
{
    uv.x += s * (4.0 * fract((uv.y * u_Resolution.y) / 2.0) - 1.0);
    return uv;
}

vec2 fault(vec2 uv, float s)
{
    //float v = (0.5 + 0.5 * cos(2.0 * pi * uv.y)) * (2.0 * uv.y - 1.0);
    float v = pow(0.5 - 0.5 * cos(2.0 * pi * uv.y), 100.0) * sin(2.0 * pi * uv.y);
    uv.x += v * s;
    return uv;
}

vec2 rnd(vec2 uv, float s)
{
    uv.x += s * (2.0 * texture(u_NoiseTexture0, uv * 0.05).x - 1.0);
    return uv;
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, -0.05, 0.05);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, -0.5, 0.5);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 0.0, 1.0);

    float t = u_Time / 10.0;

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    //float s = pow(0.5 + 0.5 * cos(2.0 * pi * t), 1000.0);
    float s = texture(u_NoiseTexture0, vec2(t * 0.2, 0.5)).r;

    uv = interlace(uv, s * coeff1);
    //uv = fault(uv, s);
    float r = texture(u_NoiseTexture1, vec2(t, 0.0)).x;
    //uv = fault(uv + vec2(0.0, fract(t * 20.0)), r) - vec2(0.0, fract(t * 20.0));
    uv = fault(uv + vec2(0.0, fract(t * 2.0)), 5.0 * sign(r) * pow(abs(r), 5.0)) - vec2(0.0, fract(t * 2.0));
    uv = rnd(uv, s * coeff2);

    vec4 color = colorSplit(uv, vec2(s * 0.02, 0.0));
    //vec2 m = texture(u_NoiseTexture1, uv).xy;
    color.rgb = mix(color.rgb, texture(u_NoiseTexture0, 0.5 * uv + t * 100.0).rgb, coeff3);

    o_FragColor = color;
}
