#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_GlitchAmount; // 1.0
uniform float u_Coeff1; // 16
uniform float u_Coeff2; // 5
uniform float u_Coeff3; // 4
uniform float u_Coeff4; // 16

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float GlitchAmount = 1.0;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

vec4 posterize(vec4 color, float numColors)
{
    return floor(color * numColors - 0.5) / numColors;
}

vec2 quantize(vec2 v, float steps)
{
    return floor(v * steps) / steps;
}

float dist(vec2 a, vec2 b)
{
    return sqrt(pow(b.x - a.x, 2.0) + pow(b.y - a.y, 2.0));
}

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    float glitchAmount = map(u_GlitchAmount, 0.0, 1.0, 0.1, 1.5);
    float coeff1 = map(u_Coeff1, 0.0, 1.0, 2.0, 32.0);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 2.0, 32.0);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 2.0, 32.0);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 2.0, 32.0);

    float amount = pow(glitchAmount, 2.0);
    vec2 pixel = 1.0 / u_Resolution.xy;
    vec4 color = texture(u_Texture, uv);

    float t = mod(mod(u_Time, amount * 100.0 * (amount - 0.5)) * 109.0, 1.0);
    vec4 postColor = posterize(color, coeff1);
    vec4 a = posterize(texture(u_Texture, quantize(uv, 64.0 * t) + pixel * (postColor.rb - vec2(.5)) * 100.0), coeff2).rbga;
    vec4 b = posterize(texture(u_Texture, quantize(uv, 32.0 - t) + pixel * (postColor.rg - vec2(.5)) * 1000.0), coeff3).gbra;
    vec4 c = posterize(texture(u_Texture, quantize(uv, 16.0 + t) + pixel * (postColor.rg - vec2(.5)) * 20.0), coeff4).bgra;

    o_FragColor = mix(
        texture(u_Texture,
                uv + amount * (quantize((a * t - b + c - (t + t / 2.0) / 10.0).rg, 16.0) - vec2(.5)) * pixel * 100.0),
            (a + b + c) / 3.0,
            (0.5 - (dot(color, postColor) - 1.5)) * amount);
}
