#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Speed; // 0.6
uniform float u_Amount; // 0.2
uniform float u_Coeff1; // 0.33

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

//2D (returns 0 - 1)
float random2d(vec2 n) {
    return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float randomRange (in vec2 seed, in float min, in float max) {
    return min + random2d(seed) * (max - min);
}

// return 1 if v inside 1d range
float insideRange(float v, float bottom, float top) {
    return step(bottom, v) - step(top, v);
}


void main() {
    float time = floor(u_Time * u_Speed * 60.0);
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    //copy orig
    vec4 outCol = texture(u_Texture, uv);

    //randomly offset slices horizontally
    float maxOffset = u_Amount/2.0;
    for (float i = 0.0; i < 10.0 * u_Amount; i += 1.0) {
        float sliceY = random2d(vec2(time , 2345.0 + float(i)));
        float sliceH = random2d(vec2(time , 9035.0 + float(i))) * 0.25;
        float hOffset = randomRange(vec2(time , 9625.0 + float(i)), -maxOffset, maxOffset);
        vec2 uvOff = uv;
        uvOff.x += hOffset;
        if (insideRange(uv.y, sliceY, fract(sliceY+sliceH)) == 1.0 ){
            outCol = texture(u_Texture, uvOff);
        }
    }

    //do slight offset on one entire channel
    float maxColOffset = u_Amount/6.0;
    float rnd = random2d(vec2(time , 9545.0));
    vec2 colOffset = vec2(randomRange(vec2(time , 9545.0),-maxColOffset,maxColOffset),
    randomRange(vec2(time , 7205.0),-maxColOffset,maxColOffset));

    vec2 c;

    if (rnd < u_Coeff1){
        c = texture(u_Texture, uv + colOffset).ra;
        outCol.r = c.x;

    }else if (rnd < 1.0 - u_Coeff1){
        c = texture(u_Texture, uv + colOffset).ga;
        outCol.g = c.x;

    } else{
        c = texture(u_Texture, uv + colOffset).ba;
        outCol.b = c.x;
    }


    o_FragColor = vec4(outCol.rgb, (outCol.a + c.y) / 2.0);
}
