#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Coeff1; // 0.08
uniform float u_Coeff2; // 8.0
uniform float u_Coeff3; // 3.0
uniform float u_Coeff4; // 0.05
uniform float u_Coeff5; // 0.05


in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}


float rng2(vec2 seed)
{
    return fract(sin(dot(seed * floor(u_Time * 12.), vec2(127.1,311.7))) * 43758.5453123);
}

float rng(float seed)
{
    return rng2(vec2(seed, 1.0));
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, 0.01, 0.5);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 1.0, 10.0);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 1.0, 10.0);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 0.01, 1.0);
    float coeff5 = map(u_Coeff5, 0.0, 1.0, 0.01, 1.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;
    vec2 blockS = floor(uv * vec2(24., 9.));
    vec2 blockL = floor(uv * vec2(8., 4.));

    float r = rng2(uv);
    vec3 noise = (vec3(r, 1. - r, r / 2. + 0.5) * 1.0 - 2.0) * coeff1;

    float lineNoise = pow(rng2(blockS), coeff2) * pow(rng2(blockL), coeff3) - pow(rng(7.2341), 17.0) * 2.;

    vec4 col1 = texture(u_Texture, uv);
    vec4 col2 = texture(u_Texture, uv + vec2(lineNoise * coeff4 * rng(5.0), 0));
    vec4 col3 = texture(u_Texture, uv - vec2(lineNoise * coeff5 * rng(31.0), 0));

    o_FragColor = vec4(vec3(col1.x, col2.y, col3.z) + noise, (col1.a + col2.a + col3.a) / 3.0);
}
