#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_Coeff1; // 1.75
uniform float u_Coeff2; // 0.1
uniform float u_Coeff3; // 0.1
uniform float u_Coeff4; // 0.15


in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    float coeff1 = map(u_Coeff1, 0.0, 1.0, 0.75, 2.5);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, -1.0, 1.0);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, -1.0, 1.0);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 0.01, 1.0);

    float t = u_Time;

    vec2 r = texture(u_Texture, uv).ra;
    vec2 g = texture(u_Texture, uv - 0.025 + cos(t + uv.y + uv.x)*coeff2).ga;
    vec2 b = texture(u_Texture, uv + 0.04  + sin(t + uv.y + uv.x)*coeff3).ba;

    o_FragColor.rgb = vec3(r.x, g.x, b.x);
    o_FragColor.rgb *= coeff1;
    o_FragColor.a = (r.y + g.y + b.y) / 3.0;

    float a = t*0.25;

    o_FragColor.gb -= texture(u_NoiseTexture, (uv + length(uv) ) * mat2(cos(a), -sin(a), sin(a), cos(a))).gb*coeff4 + length(uv - 0.5);

    o_FragColor.rgb -= length(uv - 0.5) * 0.75;
}
