#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_Speed;
uniform float u_Amplitude;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

//#define AMPLITUDE 0.1
//#define SPEED 5.0

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

vec4 rgbShift( in vec2 p , in vec4 shift) {
    shift *= 2.0*shift.w - 1.0;
    vec2 rs = vec2(shift.x,-shift.y);
    vec2 gs = vec2(shift.y,-shift.z);
    vec2 bs = vec2(shift.z,-shift.x);

    vec2 r = texture(u_Texture, p-rs).xw;
    vec2 g = texture(u_Texture, p+gs).yw;
    vec2 b = texture(u_Texture, p-bs).zw;
    float a = (r.y + g.y + b.y) / 3.0;

    return vec4(r.x, g.x, b.x, a);
}

vec4 noise( in vec2 p ) {
    return texture(u_NoiseTexture, p, 0.0);
}

vec4 vec4pow( in vec4 v, in float p ) {
    // Don't touch alpha (w), we use it to choose the direction of the shift
    // and we don't want it to go in one direction more often than the other
    return vec4(pow(v.x,p),pow(v.y,p),pow(v.z,p),v.w);
}


void main() {
    float speedCoeff = map(u_Speed, 0.0, 1.0, 0.1, 16.0);
    float amplitudeCoeff = map(u_Amplitude, 0.0, 1.0, 0.01, 1.0);

    vec2 p = gl_FragCoord.xy / u_Resolution.xy;

    // Elevating shift values to some high power (between 8 and 16 looks good)
    // helps make the stuttering look more sudden
    vec4 shift = vec4pow(noise(vec2(speedCoeff*u_Time,2.0*speedCoeff*u_Time/25.0 )),8.0)
    *vec4(amplitudeCoeff,amplitudeCoeff,amplitudeCoeff,1.0);;

    o_FragColor = rgbShift(p, shift);
}
