#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_OffsetXFactor;
uniform float u_OffsetYFactor;
uniform float u_Coeff1; // -0.02
uniform float u_Coeff2; // 200.0
uniform float u_Coeff3; // -0.01
uniform float u_Coeff4; // 2.0
uniform float u_Coeff5; // 0.05
uniform float u_Coeff6; // -0.2
uniform float u_Coeff7; // 0.43
uniform float u_Coeff8; // 0.43

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float offsetXFactor = map(u_OffsetXFactor, 0.0, 1.0, 0.0, 0.25);
    float offsetYFactor = map(u_OffsetYFactor, 0.0, 1.0, 0.0, 0.25);
    float coeff1 = map(u_Coeff1, 0.0, 1.0, -0.5, 0.5);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 1.0, 800.0);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, -0.5, 0.5);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 1.0, 800.0);
    float coeff5 = map(u_Coeff5, 0.0, 1.0, -0.5, 0.5);
    float coeff6 = map(u_Coeff6, 0.0, 1.0, -0.5, 0.5);
    float coeff7 = map(u_Coeff7, 0.0, 1.0, 0.0, 1.0);
    float coeff8 = map(u_Coeff8, 0.0, 1.0, 0.0, 1.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    vec4 moviecol;

    vec2 uvOffset = texture(u_NoiseTexture, vec2(u_Time*5.0)).rg;
    uvOffset.x *= offsetXFactor; // 0.02
    uvOffset.y *= offsetYFactor; // 0.052

    vec2 r = texture(u_Texture, uv + uvOffset + vec2(coeff1*texture(u_NoiseTexture, vec2(uv.x,uv.y/coeff2 + u_Time*5.0)).r, (tan(sin(u_Time)) * 0.6 ) * coeff5) ).ra;
    vec2 g = vec4(texture(u_Texture, uv + uvOffset)).ga;
    vec2 b = texture(u_Texture, uv / uvOffset + vec2(coeff3*texture(u_NoiseTexture, vec2(uv.x/coeff4,uv.y + u_Time*5.0)).r, coeff6) ).ba;

    moviecol.r = r.x;
    moviecol.g = g.x;
    moviecol.b = b.x;
    moviecol.a = (r.y + g.y + b.y) / 3.0;


    moviecol.rgb = mix(moviecol.rgb, vec3(dot(moviecol.rgb, vec3(coeff7))), coeff8);

    o_FragColor = vec4(moviecol);
}
