#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Amount; // 0.035
uniform float u_Speed; // 2.5
uniform float u_RatioRed; // 0.0
uniform float u_RatioBlue;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float amount = map(u_Amount, 0.0, 1.0, 0.001, 0.1);
    float speed = map(u_Speed, 0.0, 1.0, 0.1, 5.0);
    float ratioRed = map(u_RatioRed, 0.0, 1.0, 0.0, 1.0);
    float ratioBlue = map(u_RatioBlue, 0.0, 1.0, 0.0, 1.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;
    vec2 uvRed = uv;
    vec2 uvBlue = uv;
    float s = abs(sin(u_Time * speed)) * amount;
    uvRed.x += s * (1.0 - ratioRed);
    uvRed.y += s * ratioRed;
    uvBlue.x -= s * (1.0 - ratioBlue);
    uvBlue.y -= s * ratioBlue;

    o_FragColor =  texture(u_Texture, uv);

    vec2 r = texture(u_Texture, uvRed).ra;
    vec2 b = texture(u_Texture, uvBlue).ba;

    o_FragColor.r = r.x;
    o_FragColor.b = b.x;
    o_FragColor.a = (o_FragColor.a + r.y + b.y) / 3.0;

}
