#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Intensity; // 4.0
uniform float u_Coeff1; // 1.0
uniform float u_Coeff2; // 1.0

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float rand(vec2 co)
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}


void main() {
    float intensity = map(u_Intensity, 0.0, 1.0, 1.0, 32.0);
    float coeff1 = map(u_Coeff1, 0.0, 1.0, -1.0, 1.0);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, -1.0, 1.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    //tweak the factor (4.0) to change the intensity of the glitch effect
    float rnd = rand(vec2(u_Time)) * intensity;

    if (mod(gl_FragCoord.y, rnd) > rnd / 8.0)
    {
        float of = sin(gl_FragCoord.y / rnd * u_Time) * rnd * 2.0;
        uv = vec2(gl_FragCoord.x + of, gl_FragCoord.y + of / 2.0) / u_Resolution.xy;

        o_FragColor = texture(u_Texture, uv);

        vec2 r = texture(u_Texture, uv + of * rnd * vec2((0.005 + coeff2 * 0.01) * coeff1, 0.005 * coeff2)).ra;
        vec2 b = texture(u_Texture, uv - of * rnd * vec2(0.005 *(1.0 - coeff2), (0.005 + coeff1 * 0.01) * coeff2)).ba;

        o_FragColor.r *= r.x;
        o_FragColor.g *= o_FragColor.g;
        o_FragColor.b *= b.x;
        o_FragColor *= 1.5;

        o_FragColor.a = (r.y + o_FragColor.a + b.y) / 3.0;
    } else
    {
        uv = vec2(gl_FragCoord.x - rnd * 4.0, gl_FragCoord.y + rnd / 2.0) / u_Resolution.xy;

        o_FragColor = texture(u_Texture, uv);

        vec2 r = texture(u_Texture, uv + rnd * vec2(0.05 * (1.0 - coeff2), 0.05 * (1.0 - coeff1)) * coeff1).ra;
        vec2 b = texture(u_Texture, uv - rnd * vec2(0.05 * coeff1, 0.05 * coeff2) * coeff2).ba;

        o_FragColor.r *= r.x;
        o_FragColor.g *= o_FragColor.g;
        o_FragColor.b *= b.x;
        o_FragColor *= 1.5;

        o_FragColor.a = (r.y + o_FragColor.a + b.y) / 3.0;
    }
}
