#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Coeff1; // = 240;// 20.0..480.0
uniform float u_Coeff2; // = 0.5;// 0.0..1.0

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

//random hash
vec4 hash42(vec2 p){

    vec4 p4 = fract(vec4(p.xyxy) * vec4(443.8975,397.2973, 491.1871, 470.7827));
    p4 += dot(p4.wzxy, p4+19.19);
    return fract(vec4(p4.x * p4.y, p4.x*p4.z, p4.y*p4.w, p4.x*p4.w));
}


float hash( float n ){
    return fract(sin(n)*43758.5453123);
}

// 3d noise function (iq's)
float n( in vec3 x ){
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f*f*(3.0-2.0*f);
    float n = p.x + p.y*57.0 + 113.0*p.z;
    float res = mix(mix(mix( hash(n+  0.0), hash(n+  1.0),f.x),
    mix( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y),
    mix(mix( hash(n+113.0), hash(n+114.0),f.x),
    mix( hash(n+170.0), hash(n+171.0),f.x),f.y),f.z);
    return res;
}

//tape noise
float nn(vec2 p){


    float y = p.y;
    float s = u_Time*2.;

    float v = (n( vec3(y*.01 +s, 			1., 1.0) ) + .0)
    *(n( vec3(y*.011+1000.0+s, 	1., 1.0) ) + .0)
    *(n( vec3(y*.51+421.0+s, 	1., 1.0) ) + .0)
    ;
    //v*= n( vec3( (fragCoord.xy + vec2(s,0.))*100.,1.0) );
    v*= hash42(   vec2(p.x +u_Time*0.01, p.y) ).x +.3 ;


    v = pow(v+.3, 1.);
    if(v<.7) v = 0.;  //threshold
    return v;
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, 20.0, 480.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    vec4 texCol = texture(u_Texture, uv);

    float linesN = coeff1; //fields per seconds
    float one_y = u_Resolution.y / linesN; //field line
    uv = floor(uv*u_Resolution.xy/one_y)*one_y;

    float col =  nn(uv);

    o_FragColor = vec4(mix(texCol.rgb, vec3(col), u_Coeff2), texCol.a);
}
