#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Coeff1; // = 0.02;// -0.5..0.5
uniform float u_Coeff2; // = 0.016;// 0.0..0.1
uniform float u_Coeff3; // = 0.016;// 0.0..0.1
uniform float u_Coeff4; // = 30;// 1..100
uniform float u_Coeff5; // = 0.6;// 0.0..1.0

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float rand () {
    return fract(sin(u_Time)*1e4);
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, -0.5, 0.5);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 0.0, 0.1);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 0.0, 0.1);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 1.0, 100.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;

    vec2 uvR = uv;
    vec2 uvB = uv;

    uvR.x = uv.x * 1.0 - rand() * coeff2;
    uvB.y = uv.y * 1.0 + rand() * coeff3;

    //
    if(uv.y < rand() && uv.y > rand() -0.1 && sin(u_Time) < 0.0)
    {
        uv.x = (uv + coeff1 * rand()).x;
    }

    //
    vec4 c;

    vec2 r = texture(u_Texture, uvR).ra;
    vec2 g = texture(u_Texture, uv).ga;
    vec2 b = texture(u_Texture, uvB).ba;

    c.r = r.x;
    c.g = g.x;
    c.b = b.x;
    c.a = (r.y + g.y + b.y) / 3.0;

    //
    float scanline = sin( uv.y * 800.0 * rand())/coeff4;
    c.rgb *= 1.0 - scanline;

    //vignette
    float vegDist = length(( 0.5 , 0.5 ) - uv);
    c.rgb *= 1.0 - vegDist * u_Coeff5;

    o_FragColor = c;
}
