#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Coeff1;// = 0.95;// 0.0..1.0
uniform float u_Coeff2;// = 0.005;// 0.0..0.2
uniform float u_Coeff3;// = 0.007;// 0.0..0.2
uniform float u_Coeff4;// = 0.0011;// 0.0..0.2
uniform float u_Coeff5;// = 0.003;// 0.0..0.2
uniform float u_Coeff6;// = 0.001;// 0.0..0.2
uniform float u_Coeff7;// = 0.001;// 0.0..0.2

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float rand(float seed){
    return fract(sin(dot(vec2(seed) ,vec2(12.9898,78.233))) * 43758.5453);
}

vec2 displace(vec2 co, float seed, float seed2) {
    vec2 shift = vec2(0);
    if (rand(seed) > 0.5) {
        shift += 0.1 * vec2(2. * (0.5 - rand(seed2)));
    }
    if (rand(seed2) > 0.6) {
        if (co.y > 0.5) {
            shift.x *= rand(seed2 * seed);
        }
    }
    return shift;
}

vec4 interlace(vec2 co, vec4 col) {
    if (int(co.y) % 3 == 0) {
        return col * ((sin(u_Time * 4.) * 0.1) + 0.75) + (rand(u_Time) * 0.05);
    }
    return col;
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 0.0, 0.2);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 0.0, 0.2);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 0.0, 0.2);
    float coeff5 = map(u_Coeff5, 0.0, 1.0, 0.0, 0.2);
    float coeff6 = map(u_Coeff6, 0.0, 1.0, 0.0, 0.2);
    float coeff7 = map(u_Coeff7, 0.0, 1.0, 0.0, 0.2);

    vec2 uv = gl_FragCoord.xy/u_Resolution.xy;

    vec2 rDisplace = vec2(0);
    vec2 gDisplace = vec2(0);
    vec2 bDisplace = vec2(0);

    if (rand(u_Time) > u_Coeff1) {
        rDisplace = displace(uv, u_Time * 2., 2. + u_Time);
        gDisplace = displace(uv, u_Time * 3., 3. + u_Time);
        bDisplace = displace(uv, u_Time * 5., 5. + u_Time);
    }

    rDisplace.x += coeff2 * (0.5 - rand(u_Time * 37. * uv.y));
    gDisplace.x += coeff3 * (0.5 - rand(u_Time * 41. * uv.y));
    bDisplace.x += coeff4 * (0.5 - rand(u_Time * 53. * uv.y));

    rDisplace.y += coeff5 * (0.5 - rand(u_Time * 37. * uv.x));
    gDisplace.y += coeff6 * (0.5 - rand(u_Time * 41. * uv.x));
    bDisplace.y += coeff7 * (0.5 - rand(u_Time * 53. * uv.x));

    // Output to screen
    vec2 rcolor = texture(u_Texture, uv.xy + rDisplace).ra;
    vec2 gcolor = texture(u_Texture, uv.xy + gDisplace).ga;
    vec2 bcolor = texture(u_Texture, uv.xy + bDisplace).ba;

    o_FragColor = interlace(gl_FragCoord.xy, vec4(rcolor.x, gcolor.x, bcolor.x, (rcolor.y + gcolor.y + bcolor.y) / 3.0));

}
