#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_StriplesWidth;
uniform float u_StriplesOpacity;
uniform float u_NoiseIntensity;
uniform float u_VignetteIntensity;
uniform float u_VerticalShift;
uniform float u_HorizontalShift;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float noise(vec2 p)
{
    float s = texture(u_NoiseTexture,vec2(1.,2.*cos(u_Time))*u_Time*8. + p*1.).x;
    s *= s;
    return s;
}

float onOff(float a, float b, float c)
{
    return step(c, sin(u_Time + a*cos(u_Time*b)));
}

float ramp(float y, float start, float end)
{
    float inside = step(start,y) - step(end,y);
    float fact = (y-start)/(end-start)*inside;
    return (1.-fact) * inside;

}

float stripes(vec2 uv)
{

    float noi = noise(uv*vec2(0.5,1.) + vec2(1.,3.)) * u_StriplesOpacity;
    return ramp(mod(uv.y*4. + u_Time/2.+sin(u_Time + sin(u_Time*0.63)),1.),0.0,u_StriplesWidth)*noi;
}

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

vec4 getVideo(vec2 uv)
{
    float verticalShift = map(u_VerticalShift, 0.0, 1.0, 0.0, 2.0);
    float horizontalShift = map(u_HorizontalShift, 0.0, 1.0, 0.0, 2.0);;

    vec2 look = uv;
    float window = 1./(1.+20.*(look.y-mod(u_Time/4.,1.))*(look.y-mod(u_Time/4.,1.)));
    look.x = look.x + (sin(look.y*10. + u_Time)/50.*onOff(4.,4.,.3)*(1.+cos(u_Time*80.))*window) * horizontalShift;
    float vShift = (0.4*onOff(2.,3.,.9)*(sin(u_Time)*sin(u_Time*20.) +
        (0.5 + 0.1*sin(u_Time*200.)*cos(u_Time)))) * verticalShift;
    look.y = mod(look.y + vShift, 1.);

    return texture(u_Texture,look);
}

vec2 screenDistort(vec2 uv)
{
    uv -= vec2(.5,.5);
    uv = uv*1.2*(1./1.2+2.*uv.x*uv.x*uv.y*uv.y);
    uv += vec2(.5,.5);
    return uv;
}

void main() {
    vec2 uv = gl_FragCoord.xy / u_Resolution.xy;
    uv = screenDistort(uv);
    vec4 video = getVideo(uv);
    float vigAmt = (3.+.3*sin(u_Time + 5.*cos(u_Time*5.))) * u_VignetteIntensity;
    float vignette = (1.-vigAmt*(uv.y-.5)*(uv.y-.5))*(1.-vigAmt*(uv.x-.5)*(uv.x-.5));

    video.rgb += stripes(uv);
    video.rgb += noise(uv*2.) * u_NoiseIntensity;
    video.rgb *= vignette;
    video.rgb *= (12.+mod(uv.y*30.+u_Time,1.))/13.;

    o_FragColor = video;
}
