#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_Range;
uniform float u_NoiseQuality;
uniform float u_NoiseIntensity;
uniform float u_OffsetIntensity;
uniform float u_ColorOffsetIntensity;

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float range = 0.05;
float noiseQuality = 250.0;
float noiseIntensity = 0.0088;
float offsetIntensity = 0.02;
float colorOffsetIntensity = 1.3;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

float rand(vec2 co)
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float verticalBar(float pos, float uvY, float offset)
{
    float edge0 = (pos - range);
    float edge1 = (pos + range);

    float x = smoothstep(edge0, pos, uvY) * offset;
    x -= smoothstep(pos, edge1, uvY) * offset;
    return x;
}

void main() {
    range = map(u_Range, 0.0, 1.0, 0.0, 0.5);
    noiseQuality = map(u_NoiseQuality, 0.0, 1.0, 12.5, 250.0);
    noiseIntensity = map(u_NoiseIntensity, 0.0, 1.0, 0.0001, 0.01);
    offsetIntensity = map(u_OffsetIntensity, 0.0, 1.0, 0.005, 0.1);
    colorOffsetIntensity = map(u_ColorOffsetIntensity, 0.0, 1.0, 0.0, 4.0);

    vec2 uv = gl_FragCoord.xy / u_Resolution;

    for (float i = 0.0; i < 0.71; i += 0.1313)
    {
        float d = mod(u_Time * i, 1.7);
        float o = sin(1.0 - tan(u_Time * 0.24 * i));
        o *= offsetIntensity;
        uv.x += verticalBar(d, uv.y, o);
    }

    float uvY = uv.y;
    uvY *= noiseQuality;
    uvY = float(int(uvY)) * (1.0 / noiseQuality);
    float noise = rand(vec2(u_Time * 0.00001, uvY));
    uv.x += noise * noiseIntensity;

    vec2 offsetR = vec2(0.006 * sin(u_Time), 0.0) * colorOffsetIntensity;
    vec2 offsetG = vec2(0.0073 * (cos(u_Time * 0.97)), 0.0) * colorOffsetIntensity;

    vec2 r = texture(u_Texture, uv + offsetR).ra;
    vec2 g = texture(u_Texture, uv + offsetG).ga;
    vec2 b = texture(u_Texture, uv).ba;



    o_FragColor = vec4(r.x, g.x, b.x, (r.y + g.y + b.y) / 3.0);
}
