#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp
uniform sampler2D u_NoiseTexture;

uniform float u_Coeff1;// = -0.01;// -0.2..0.2
uniform float u_Coeff2;// = 0.6;// 0.0..1.0
uniform float u_Coeff3;// = 0.001;// 0.0..0.01
uniform float u_Coeff4;// = 0.003;// 0.0..0.01

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, -0.2, 0.2);
    float coeff3 = map(u_Coeff3, 0.0, 1.0, 0.0, 0.01);
    float coeff4 = map(u_Coeff4, 0.0, 1.0, 0.0, 0.01);

    vec2 uv = gl_FragCoord.xy / u_Resolution;

    vec4 moviecol;

    vec2 uvOffset = texture(u_NoiseTexture, vec2(u_Time*5.0)).rg;
    uvOffset.x *= coeff3;
    uvOffset.y *= coeff4;

    vec2 r = texture(u_Texture, uv + uvOffset ).ra;
    vec2 g = vec4(texture(u_Texture, uv + uvOffset)).ga;
    vec2 b = texture(u_Texture, uv + uvOffset + vec2(coeff1*texture(u_NoiseTexture, vec2(uv.x/100.0,uv.y + u_Time*5.0)).r, 0) ).ba;

    moviecol.r = r.x;
    moviecol.g = g.x;
    moviecol.b = b.x;


    moviecol.rgb = mix(moviecol.rgb, vec3(dot(moviecol.rgb, vec3(.33))), u_Coeff2);
    moviecol.a = (r.y + g.y + b.y) / 3.0;

    o_FragColor = vec4(moviecol);
}
