#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Resolution;
uniform sampler2D u_Texture; // clamp

uniform float u_Coeff1;// = 0.3;// 0.0..2.0
uniform float u_Coeff2;// = 0.01;// 0.0..0.15

in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    float coeff1 = map(u_Coeff1, 0.0, 1.0, 0.0, 2.0);
    float coeff2 = map(u_Coeff2, 0.0, 1.0, 0.0, 0.15);

    float amount = sin(u_Time) * 0.1;

    // uv coords
    vec2 uv = gl_FragCoord.xy / u_Resolution;

    amount *= coeff1;
    float split = 1. - fract(u_Time / 2.);
    float scanOffset = coeff2;
    vec2 uv1 = vec2(uv.x + amount, uv.y);
    vec2 uv2 = vec2(uv.x, uv.y + amount);

    if (uv.y > split) {
        uv.x += scanOffset;
        uv1.x += scanOffset;
        uv2.x += scanOffset;
    }

    vec2 r = texture(u_Texture, uv1).ra;
    vec2 g = texture(u_Texture, uv).ga;
    vec2 b = texture(u_Texture, uv2).ba;

    o_FragColor = vec4(r.x, g.x, b.x, (r.y + g.y + b.y) / 3.0);
}
